Site Integrator
***************

Enables integration of external web sites into a Drupal site.

===============================


CONTENTS OF THIS FILE
---------------------

 * Introduction
 * Requirements
 * Installation
 * Configuration
 * Technical aspects
 * Maintainers


INTRODUCTION
------------

The Site Integrator module enables the integration of external websites into a
Drupal site using either an iframe or by merging HTML content directly.

It is common to integrate web tools developed by third parties (not using
Drupal) into a Drupal site. The simplest method is to create a page with an
iframe that uses the external site’s URL as the source. However, this approach
has several drawbacks:
* The initial iframe size must be set manually.
* When the iframe content changes, the iframe needs to be resized accordingly.
* Retrieving the size of the iframe content is often not possible due to
  security restrictions.
* If the external site requires restricted access, its URL must be publicly
  accessible to use within an iframe, meaning you’ll need to manage access
  restrictions on that external site, with no control on the Drupal side. While
  it’s possible to set up a complex single-sign-on (SSO) system to avoid
  multiple logins, this may not be straightforward or supported natively by the
  external site.
* There is no way to dynamically customize the external site’s look and feel to
  match your Drupal site. Customizing the external site directly may not be
  convenient, especially if it is also accessed by other websites with different
  designs.
* The iframe isolates the external site from Drupal, making direct interaction
 difficult. For instance, you can’t easily create a widget in Drupal that
 interacts with the external site, such as loading data, selecting options, or
 performing actions and vice versa.

## Features

With this module, you can still use a classic iframe to integrate an external
site while accepting the limitations mentioned above. However, you can also opt
for an enhanced iframe solution that addresses these issues. By using an iframe
with "local" content (i.e., the external site's content is fetched server-side
and injected into the iframe via JavaScript rather than using a source URL),
several advantages emerge. You can handle iframe content changes, manage access
restrictions (i.e., the external site can be accessible only to the web server
running Drupal, with Drupal acting as a gateway), and even modify the external
site's content on-the-fly to better align with Drupal’s look and feel.

Furthermore, this module allows for complete iframe removal, enabling direct
integration of HTML content into a page. This provides real, seamless
interaction without the overhead of an iframe, which can consume more memory due
to environment isolation. However, this approach may cause conflicts with some
external sites, particularly with JavaScript libraries, CSS, or HTML element
identifiers. Since the HTML document structure is altered, certain
functionalities may not work as expected. In most cases, on-the-fly
modifications will be necessary to make this approach successful.


REQUIREMENTS
------------

This module has no specific requirements.


INSTALLATION
------------

 * Install as you would normally install a contributed Drupal module. Visit
   https://www.drupal.org/node/1897420 for further information.


CONFIGURATION
-------------

After enabling the module, navigate to the site’s "Configuration" page. Under
the "External Data" section, you will find a link to list, add, or edit
"Integrated Sites.".

An "integrated site" consists of 4 required elements:
* a "label" to identify the integration (administration).
* a "base URL" corresponding to the base URL of the external site to integrate.
  This URL must include the HTTP scheme. It will only be visible to users if
  the "integration mode" is set to a regular external site iframe. In ther other
  cases, the URL should not be visible.
* a "Drupal path" corresponding to the part of the URL after the Drupal site
  base URL that will lead to the site integration page. It should start with a
  slash (which would be automatically added if missing). The path must not be
  already in use by an other part of the Drupal site.
* an "integration mode" wich sets how the site will be technically integrated.
  Each mode of integration has its own strength and drwabacks:
  * "Merge site content": the HTML content of the external site will be fetched
    by the Drupal server and altered to remove some elements like "DOCTYPE", 
    "<html>", "<head>", "meta", "title", and "<body>" tags for instance. The
    resulting HTML content will be then included into the integrated site Drupal
    page.
    This approach may cause conflicts with some external sites, particularly
    with JavaScript libraries, CSS, or HTML element identifiers. Since the HTML
    document structure is altered, certain functionalities may not work as
    expected. In most cases, on-the-fly modifications will be necessary to make
    this approach successful.
  * "Integrate into an IFrame as local content": the HTML content of the
    external site will be fetched by the Drupal server and then injected into an
    iframe in the site integration page via JavaScript rather than using a
    source URL. The integrated site will run in a semi-isolated environment (no
    CSS, Javascript or HTML element conflicts) but will still be able to
    communicate with the Drupal page, for instance to automatically update the
    iframe size when the content changes dynimcally.
    It is also possible to use this mode when the site to include is only
    visible from the Drupal sever but not from outside: it becomes possible to
    manage access restriction to such a site using Drupal as a proxy/gateway.
    The external site content can also be modified on-the-fly to remove extra
    menus, alter CSS to adjust the look-and-feel, or add custom Javascript files
    to interact with the Drupal side for example.
  * "Integrate into an IFrame as a regular external site": the mode just creates
    a regular iframe using the given base URL as source. The iframe size can not
    be adjusted automatically.

Integrated sites have other options:
* "Passthrough URL": it provides the list of paths of the integrated site that
  will not be integrated into a Drupal page but rather served as they are on the
  external site. This list is especially usefull for HTML content that will be
  served through AJAX calls by the external site to avoid alterations by the
  site integrator.
  Note: it is useless to add non-HTML content path to that list as such contents
  are always served without alteration (ie. images, css, javascripts, json, text
  files, ...).
* "Redirection paths and URLs": it allows to redirect to a given Drupal site
  path or an external URL when a given HTTP code on the integrated site is met.
  For instance, you can redirect "404 - not found" integrated site pages to
  Drupal's 404 pages. It is also possible to handle sets of codes like 3xx, 4xx
  or 5xx to have them all redirected to a same page. If generic codes are used
  (ie. "4xx") and also specific codes (ie. "404"), the specific codes will
  always have a higher priority.
  There 2 additional special codes: "disabled" used when the integrated site is
  disabled and "forbidden" when Drupal forbids the access to the integrated site
  page to current Drupal user (not to be confused with a 403 status code
  returned by the integrated site itself). Redirections will redirect the Drupal
  page (and not the integrated site page).
* "Filters": filters can be used to alter on-the-fly the integrated site content
  before it is processed by the web browser. It can help to avoid conflicts when
  merging external site HTML content to a Drupal page but it can also be used to
  customize the external site (CSS changes, HTML element removal or additions,
  custom supplementary Javascript to interact with the parent Drupal site).
* "Role-based access restriction": it is possible to list the Drupal roles that
  are allowed to access the Drupal site integration. If no role is set, all
  users are allowed to access to the given integrated site. If one or more roles
  are set, only user with those roles will be allowed to access the integrated
  site; other users will be either redirected to a given "forbidden" page if set
  in the "Redirection paths and URLs" parameter or will be displayed the Drupal
  "access denied" page.

To integrate a site, you should start by using the iframe with your site as a
regular external site and see how it behaves. If it works well, then try as
local content for the iframe. See if elements are missing and have a look to the
Javascript logs. It may help you understand the problems you may meet. It's
often a problem of incorrect URLs. Incorrect relative URL should be tricked
using other "site integrations" with different paths to meet the needs.
You may also meet "CORS" issues which could be solved by altering the URLs
on-the-fly using regexp filters. If the site supports "infinitie scrolling", you
will have to specify a fixed site height on the site integration settings page.
If the site integrates well with a "local iframe", you can try to use the HTML
merging technic. Again, you will have to investigate the Javascript/HTML logs
for issues. There are often CSS, Javascript or HTML element conflicts that can
be solved using filters. Sometimes, you may also have to alter CSS or Javascript
files: you will jave to make a local copy of those files from the external site
on your Drupal site and modify them there "statically". Then, use filters to
replace the external site versions with yours.


TECHNICAL ASPECTS
-----------------

When using "merge" or "internal iframe" modes, only HTML content will be
altered. Other types of content will be passed through, like images, CSS files,
Javascript files and such.

Could any site be included? The answer is "technically" yes but it's not always
easy and sometimes requires many alterations. Therefore, if it requires too much
efforts, some sites can be considered as not integrable.

WARNING!
Do not grant permission to create integrated site to untrusted users: the site
integration system allows to access to sites that are visible from the Drupal
server, which could include intranet servers that are not visible by external
users but would become visible through site intgration.


MAINTAINERS
-----------

Current maintainers:
  * Valentin Guignon (guignonv) - https://www.drupal.org/u/guignonv
