<?php

namespace Drupal\site_integrator\Controller;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Url;
use Drupal\site_integrator\Entity\IntegratedSiteInterface;

/**
 * Provides a listing of integrated sites.
 *
 * @ingroup site_integrator
 */
class IntegratedSiteListBuilder extends ConfigEntityListBuilder {

  /**
   * {@inheritdoc}
   */
  protected function getModuleName() {
    return 'site_integrator';
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['label'] = $this->t('Integrated site');
    $header['path'] = $this->t('Path');
    $header['base_url'] = $this->t('URL');
    $header['mode'] = $this->t('Mode');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $link_generator = \Drupal::service('link_generator');
    $path_url_str = $entity->getPath();
    $path_url = Url::fromUri('base:/' . $path_url_str);
    $base_url_str = $entity->getBaseUrl();
    $base_url = Url::fromUri($base_url_str);
    $mode = $entity->getIntegrationMode();
    switch ($mode) {
      case IntegratedSiteInterface::INTEGRATION_MODE_MERGE:
        $mode = $this->t('Merge');
        break;

      case IntegratedSiteInterface::INTEGRATION_MODE_INTERNAL_IFRAME:
        $mode = $this->t('Local');
        break;

      case IntegratedSiteInterface::INTEGRATION_MODE_EXTERNAL_IFRAME:
        $mode = $this->t('External');
        break;

      default:
        $mode = $this->t('Unknown');
        break;

    }

    $row['label'] = $entity->label();
    $row['path'] = $link_generator->generate($path_url_str, $path_url);
    $row['base_url'] = $link_generator->generate($base_url_str, $base_url);
    $row['mode'] = $mode;

    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getOperations(EntityInterface $entity) {
    if (($entity instanceof IntegratedSiteInterface)
      && (!$entity->isEditable())
    ) {
      return [];
    }
    return parent::getOperations($entity);
  }

}
